/*
 * Manages visibility of fields in the CWL tab in WooCommerce settings.
 * 
 * @global CWL defined by wp_localize_script
 * @global jQuery
 */
(function($){
    $(function(){
        // mode settings
        var $pbdsTable = $('.cwl-points-mode').eq(0).closest('table.form-table');
        var $tbdsTable = $('.cwl-tiers-mode').eq(0).closest('table.form-table');
        
        // toggle based on initialsettings
        toggleMode();
        
        $('.cwl-mode-select').on('change', toggleMode);
        
        // add another tier button
        var $addTierButton = $('.cwl-add-tier-button');
        $addTierButton.on('click', addTierRow);
        
        // bind delete tiers buttons
        $tbdsTable.on('click', '.remove-tier', deleteTierRow);
        
        // add currency symbol to some inputs
        var $currInputs = $('.cwl-currency-input');
        $currInputs.each(addCurrencySymbol);


        /**
         * Toggles the visisble fields based on the selected mode.
         * @returns {undefined}
         */
        function toggleMode(){
            var mode = $('.cwl-mode-select:checked').val();
            if(mode == 'points'){
                $tbdsTable.hide();
                $tbdsTable.prev('p').hide();
                $tbdsTable.prev('p').prev('h2').hide();
                
                $pbdsTable.show();
                $pbdsTable.prev('h2').show();
            }else{
                $pbdsTable.hide();
                $pbdsTable.prev('h2').hide();
                
                $tbdsTable.show();
                $tbdsTable.prev('p').show();
                $tbdsTable.prev('p').prev('h2').show();
            }
        }
        
        /**
         * Add another input row for adding tiers.
         * @returns {undefined}
         */
        function addTierRow(){
            var $row = $addTierButton.closest('tr').prev('tr');
            var $newRow = $row.clone();
            
            var oldLabel = $row.find('label').text();
            var newNumber = parseInt(oldLabel.match(/\d+/)[0])+1;
            var newLabel = oldLabel.replace(/\d+/, newNumber);
            $newRow.find('label').text(newLabel);
            
            // clear values and update indexes
            $newRow.find('input').each(function(){
                var $this = $(this);
                
                $this.val('');
                $this.attr('name', $this.attr('name').replace(/\[\d+\]/, '['+newNumber+']'));
            });
            

            $row.after($newRow);
        }
        
        /**
         * Deletes the tier row that delete button was clicked on.
         * @returns {undefined}
         */
        function deleteTierRow(){
            $(this).closest('tr').remove();
            
            refreshTierNumbers();
        }
        
        /**
         * Refreshes tier number. Useful when a row is deleted, for example.
         * @returns {undefined}
         */
        function refreshTierNumbers(){
            $tbdsTable.find('.cwl-tier-row').each(function(index){
                console.log(index);
                var $row = $(this);
                
                var oldLabel = $row.find('label').text();
                var newNumber = index+1;
                var newLabel = oldLabel.replace(/\d+/, newNumber);
                $row.find('label').text(newLabel);
            });
        }
        
        /**
         * Adds a currency symbol to currency inputs.
         * @param {Number} index
         * @param {DOMEelement} input
         * @returns {undefined}
         */
        function addCurrencySymbol(index, input){
            var $input = $(input);
            $input.before('<span class="cwl-currency-input-synbol">'+CWL.currencySymbol+'</span>');
        }
    });
})(jQuery);
