<?php

/**
 * Provides init code for admin panel sections for this plugin.
 * 
 * Globals defined in codup-woo-loyalty.php:
 * @global string CWL_SLUG contains a string ID for this plugin.
 * @global string CWL_PLUGIN_DIR_URL the url to this plugins directory
 * @global string CWL_VERSION the plugins version number
 */
class CodupWooLoyaltyAdmin 
{
    /**
     * Default values for all the settings.
     * @var string[]|int[]|float[]
     */
    private static $optDefaults = array(
        
        //_________SPIRINT 1 :Support General Settings Discount_______________________//
        CodupWooLoyalty::OPT_DISCOUNT_MODE => CodupWooLoyalty::MODE_POINTS,
        CodupWooLoyalty::OPT_POINT_VALUE => 0.02,
        CodupWooLoyalty::OPT_POINT_DISCOUNT_VALUE => 0.01,
        CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN => 365,
        CodupWooLoyalty::OPT_MAX_POINT_DISCOUNT => 10,
        CodupWooLoyalty::OPT_ADD_POINTS_STATUS => 'wc-completed',
        CodupWooLoyalty::OPT_DISCOUNT_TIERS => array(array('from' => 0, 'to' => 1000, 'discount' => 1)),

    );
    
    /**
     * Helps validate admin input fields.
     * @var CodupWooLoyaltyValidator 
     */
    private $validator;
    
    /**
     * Hold a reference to tier type, so we don't register it twice.
     * @var CodupWooLoyaltyTiersFields
     */
    private static $tierType = null;
    
    /**
     * Initiates the validator. Initialized the tier field type.
     */
    public function __construct()
    {
        $this->validator = new CodupWooLoyaltyValidator();
        
        if(self::$tierType == null){
            self::$tierType = new CodupWooLoyaltyTiersFields();
            self::$tierType->hookIntoWP();
        }
    }
    
    /**
     * Hooks into WP to create new section in WooCommerce settings.
     */
    public function hookIntoWP()
    {
        // add fields on woocommerce category page
        $mode = get_option(CodupWooLoyalty::OPT_DISCOUNT_MODE, CodupWooLoyalty::MODE_POINTS);
        if($mode == CodupWooLoyalty::MODE_POINTS)
        {
            add_action( 'product_cat_edit_form_fields', array($this,'getEditPageCategoryPointsModeFields'),10,2);
        }
        else
        {
            add_action( 'product_cat_edit_form_fields', array($this,'addCustomFieldsOnCatPage'),10,2);
        }
        add_action( 'product_cat_add_form_fields', array($this,'addCustomFieldsOnCatPage'),10,2);
        add_action( 'edited_product_cat', array($this,'saveCustomFieldsOnCatPage'));  
        add_action( 'create_product_cat', array($this,'saveCustomFieldsOnCatPage'));
        
        // add tab on woocommerce product page
        add_action('woocommerce_product_write_panel_tabs', array( $this, 'codup_woo_loyalty_tab'),10,2);
        add_action('woocommerce_product_data_panels', array( $this, 'codup_woo_loyalty_tab_fields'),10,2);
        add_action('woocommerce_process_product_meta', array( $this, 'saveCustomFieldsOnProductPage'));

        // filters
        add_filter('woocommerce_settings_tabs_array', array($this, 'addSettingsTab'), 50);
        add_filter('plugin_action_links_'.CWL_PLUGIN_BASENAME, array($this, 'addActionLinks'));
        
        // settings validation
        add_filter('woocommerce_admin_settings_sanitize_option_'.CodupWooLoyalty::OPT_POINT_VALUE, array($this->validator, 'validatePointValue'), 10, 3);
        add_filter('woocommerce_admin_settings_sanitize_option_'.CodupWooLoyalty::OPT_POINT_DISCOUNT_VALUE, array($this->validator, 'validatePointDiscountValue'), 10, 3);
        add_filter('woocommerce_admin_settings_sanitize_option_'.CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN, array($this->validator, 'validatePointDiscountValue'), 10, 3);
        add_filter('woocommerce_admin_settings_sanitize_option_'.CodupWooLoyalty::OPT_MAX_POINT_DISCOUNT, array($this->validator, 'validateMaxPointDiscount'), 10, 3);
        add_filter('woocommerce_admin_settings_sanitize_option_'.CodupWooLoyalty::OPT_ADD_POINTS_STATUS, array($this->validator, 'validateAddPointsStatus'), 10, 3);
        add_filter('woocommerce_admin_settings_sanitize_option_'.CodupWooLoyalty::OPT_DISCOUNT_TIERS, array($this->validator, 'validateDiscountTier'), 10, 3);
        
        // WP actions
        add_action('init', array($this, 'addSettingDefaults'));
        
        // WC actions
        add_action('woocommerce_settings_tabs_'.CWL_SLUG, array($this, 'addSettingsTabContents'));
        add_action('woocommerce_update_options_'.CWL_SLUG, array($this, 'saveSettings'));
        
        // add JS and CSS
        add_action('admin_enqueue_scripts', array($this, 'enqueueScriptsAndStyles'));
    }
    
    /**
     * Registers the admin page JS scripts to be used on the specific tab in WooCommerce
     * settings.
     * @param string $hook
     */
    public function enqueueScriptsAndStyles($hook)
    {
        if($hook == 'woocommerce_page_wc-settings' && isset($_GET['tab']) && $_GET['tab'] == CWL_SLUG){
            wp_enqueue_style(CWL_SLUG.'_admin_css', CWL_PLUGIN_DIR_URL.'assets/css/admin.css', array(), CWL_VERSION);
            wp_enqueue_script(CWL_SLUG.'_admin', CWL_PLUGIN_DIR_URL.'assets/js/admin.js', array('jquery'), CWL_VERSION);
            
            wp_localize_script(CWL_SLUG.'_admin', 'CWL', array(
                'currencySymbol' => get_woocommerce_currency_symbol()
            ));
        }
        else if($hook == 'edit-tags.php' || $hook == 'term.php' || $hook == 'post.php')
        {
            $mode = get_option(CodupWooLoyalty::OPT_DISCOUNT_MODE, CodupWooLoyalty::MODE_POINTS);
            if($mode == CodupWooLoyalty::MODE_POINTS)
            {    
                wp_enqueue_style(CWL_SLUG.'_admin_css', CWL_PLUGIN_DIR_URL.'assets/css/admin.css', array(), CWL_VERSION);
            }
            else
            {
                wp_enqueue_style(CWL_SLUG.'_admin_css', CWL_PLUGIN_DIR_URL.'assets/css/admin.css', array(), CWL_VERSION);
                wp_enqueue_script(CWL_SLUG.'_admin', CWL_PLUGIN_DIR_URL.'assets/js/admin.js', array('jquery'), CWL_VERSION);
                
                wp_localize_script(CWL_SLUG.'_admin', 'CWL', array(
                    'currencySymbol' => get_woocommerce_currency_symbol()
                ));
            }
        }
    }
    
    /**
     * Inserts default values into WordPress options.
     */
    public function addSettingDefaults()
    {
        foreach(self::$optDefaults as $key => $defaultValue)
        {
            if(!get_option($key))
            {
                add_option($key, $defaultValue);    
            }
        }
    }
    
    /**
     * Adds the CWL tab to WooCommerce settings page.
     * @param string[] $tabs
     * @return string[]
     */
    public function addSettingsTab($tabs)
    {
        $tabs[CWL_SLUG] = __('Loyalty', CWL_SLUG);
        return $tabs;
    }
    
    /**
     * Adds the content of the CWL settings tab in WC settings page.
     */
    public function addSettingsTabContents()
    {
        woocommerce_admin_fields($this->generateSettingsFields());
    }
    
    /**
     * Saves the settings entered into the form.
     */
    public function saveSettings()
    {
        woocommerce_update_options($this->generateSettingsFields(true));
    }
    
    /**
     * Generates the WC settings fields configuration for this plugin tab.
     * @param booelan $forSaving whether we are getting the fields for saving or displaying
     * @return mixed[]
     */
    private function generateSettingsFields($forSaving = false)
    {
        echo do_shortcode("[codup_ads_top]");
        $settings = array(
            'general_title' => array(
                'title' => __('General Settings', CWL_SLUG),
                'type' => 'title',
                'id' => 'wc_settings_tab_'.CWL_SLUG.'_general_title'
            ),
            'mode' => array(
                'title' => __('Discount Mode', CWL_SLUG),
                'type' => 'radio',
                'options' => array(
                    CodupWooLoyalty::MODE_POINTS => __('Points Based Discount'),
                    CodupWooLoyalty::MODE_TIERS => __('Tiers Based Discont')
                ),
                'desc_tip' => sprintf(
                    __('<strong>Points Based Discount:</strong> each point has a 
                        specific monetary value, which can be used to reduce the 
                        price, up to a configurable maximum.
                        <br/><br/>
                        <strong>Tier Based Discount:</strong> offer a % discount 
                        depending on the amount of money spent during a configurable 
                        period.', CWL_SLUG), 
                    get_woocommerce_currency_symbol()
                ),
                'desc' => __('Select how the loyalty system applied discounts.', CWL_SLUG), 
                'id' => CodupWooLoyalty::OPT_DISCOUNT_MODE,
                'class' => 'cwl-mode-select'
            ),
            'point_lifespan' => array(
                'title' => __('Discount Liefespan', CWL_SLUG),
                'type' => 'number',
                'desc_tip' => __('<strong>Points Based Discount Mode:</strong> 
                                If the enetred amount of days pass and the user has not spent their points,
                                the points are removed from the users account.
                                Entering "0" will make the points will never expire.
                                <br/><br/>
                                <strong>Tier Based Discount Mode:</strong> 
                                Discount amount is based on money spent during 
                                the period specified here.
                                Entering "0" will base the discount on the users
                                all time spending.', CWL_SLUG), 
                'desc' => __('After how many days the user discounts expire.', CWL_SLUG), 
                'id' => CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN,
            ),
            'general_section_end' => array(
                'type' => 'sectionend',
                'id' => 'wc_settings_tab_'.CWL_SLUG.'_general_title'
            )
        );
        
        $settings = array_merge($settings, $this->getPointsModeFields());
        $settings = array_merge($settings, $this->getTiersModeFields($forSaving));
        
        return apply_filters(CWL_SLUG.'_settings', $settings);
    }
    
    /**
     * Returns the setting fields for the points based discount mode.
     */
    private function getPointsModeFields()
    {
        return array(
            'pbds_title' => array(
                'title' => __('Points Based Discount Settings', CWL_SLUG),
                'type' => 'title',
                'id' => 'wc_settings_tab_'.CWL_SLUG.'_pbds_title',
            ),
            'point_value' => array(
                'title' => __('Point Value', CWL_SLUG),
                'type' => 'text',
                'desc_tip' => sprintf(
                    __('How much money needs to be spent to receive a single point. 
                        For example, entering 0.5 will mean that for each %s spent, 
                        the user will be awarded 2 points.', CWL_SLUG), 
                    wc_price(1)
                ),
                'desc' => __('How much money needs to be spent to receive a single point.', CWL_SLUG), 
                'id' => CodupWooLoyalty::OPT_POINT_VALUE,
                'class' => 'cwl-points-mode cwl-currency-input'
            ),
            'point_discount_value' => array(
                'title' => __('Point Discount Value', CWL_SLUG),
                'type' => 'text',
                'desc_tip' => sprintf(
                    __('How much to reduce the cart total price for each point spent. 
                        For example, entering 0.01 will mean that for each point spent, 
                        the cart total will be reduced by %s.', CWL_SLUG), 
                    wc_price(0.01)
                ),
                'desc' => __('How large a discount a single point is worth.', CWL_SLUG), 
                'id' => CodupWooLoyalty::OPT_POINT_DISCOUNT_VALUE,
                'class' => 'cwl-points-mode cwl-currency-input'
            ),
            'max_point_discount' => array(
                'title' => __('Maximum Discount', CWL_SLUG),
                'type' => 'number',
                'desc_tip' => __('If the maximum discount is set to 10%, then the user will only be able to 
                                spend points up to the 10% discount, no more.', CWL_SLUG), 
                'desc' => __('%. The maximum amount if discount the user can get using points.', CWL_SLUG), 
                'id' => CodupWooLoyalty::OPT_MAX_POINT_DISCOUNT,
                'class' => 'cwl-points-mode'
            ),
            'add_point_status' => array(
                'title' => __('When to Credit Points', CWL_SLUG),
                'type' => 'select',
                'options' => wc_get_order_statuses(),
                'desc_tip' => __('If the maximum discount is set to 10%, then the user will only be able to 
                                spend points up to the 10% discount, no more.', CWL_SLUG), 
                'desc' => __('The points will be credited to the user account when their order reaches the selected status.', CWL_SLUG), 
                'id' => CodupWooLoyalty::OPT_ADD_POINTS_STATUS,
                'class' => 'cwl-points-mode'
            ),
            'pbds_section_end' => array(
                'type' => 'sectionend',
                'id' => 'wc_settings_tab_'.CWL_SLUG.'_pbds_title'
            )
        );
    }
    
    /**
     * Returns the setting fields for the tiers based discount mode.
     * @return mixed[]
     * @param booelan $forSaving whether we are getting the fields for saving or displaying
     */
    private function getTiersModeFields($forSaving = false)
    {
        $fields = array(
            'tbds_title' => array(
                'title' => __('Discount Tiers', CWL_SLUG),
                'type' => 'title',
                'desc' => __('Please enter the ranges of money the user needs to '.
                            'have spent within the discount lifespan to get the '. 
                            'discounts in the respective tiers.<br/>', CWL_SLUG), 
                'id' => 'wc_settings_tab_'.CWL_SLUG.'_tbds_title',
            ),
            'tier_header' => array(
                'type' => CodupWooLoyaltyTiersFields::FIELD_HEADER_TYPE,
                'id' => 'cwl-tiers-header'
            ),
        );
        
        $tiers = get_option(CodupWooLoyalty::OPT_DISCOUNT_TIERS);
        if($tiers == null || count($tiers) == 0)
        {
            $tiers = array(array('from' => '', 'to' => '', 'discount' => ''));
        }
        $fields['tiers'] = array(
            'title' => __('Tier %d', CWL_SLUG),
            'type' => CodupWooLoyaltyTiersFields::FIELD_TYPE,
            'id' => CodupWooLoyalty::OPT_DISCOUNT_TIERS,
            'class' => 'cwl-tiers-mode',
            'tiers' => $tiers
        );
        
        $fields['tier_footer'] = array(
            'type' => CodupWooLoyaltyTiersFields::FIELD_FOOTER_TYPE,
            'id' => 'cwl-tiers-footer'
        );
        $fields['tbds_section_end'] = array(
            'type' => 'sectionend',
            'id' => 'wc_settings_tab_'.CWL_SLUG.'_tbds_title',
        );
        
        return $fields;
    }
    
    
    //---------------------------SPIRINT 2 :Support Category Specific Discount-----------------------------------------//


    /**
     * save the category page custom fields for the tiers based discount mode.
     * save the category page custom fields for the points based discount mode.
     * @param $term_id getting the id of category 
     */
    public function saveCustomFieldsOnCatPage( $term_id ) 
    {
        // Save the taxonomy value.            
        $term_meta = get_term_meta($term_id);
        $mode = get_option(CodupWooLoyalty::OPT_DISCOUNT_MODE, CodupWooLoyalty::MODE_POINTS);

        if($mode == CodupWooLoyalty::MODE_POINTS)
        {    
            if(isset($_POST[CodupWooLoyalty::OPT_CATEGORY_POINTS_VALUE]) && $_POST[CodupWooLoyalty::OPT_CATEGORY_POINTS_VALUE] >= 0 && is_numeric($_POST[CodupWooLoyalty::OPT_CATEGORY_POINTS_VALUE]))
            {
                update_term_meta($term_id, CodupWooLoyalty::OPT_CATEGORY_POINTS_VALUE, $_POST[CodupWooLoyalty::OPT_CATEGORY_POINTS_VALUE]);
            }
            else if($_POST[CodupWooLoyalty::OPT_CATEGORY_POINTS_VALUE]=='' || empty($_POST[CodupWooLoyalty::OPT_CATEGORY_POINTS_VALUE]))
            {   
                update_term_meta($term_id, CodupWooLoyalty::OPT_CATEGORY_POINTS_VALUE, 0);
            }
            else
            {
                WC_Admin_Meta_Boxes::add_error('Invalid Data Entered In Point Discount');
            }
            if(isset($_POST[CodupWooLoyalty::OPT_CATEGORY_MAX_POINT_DISCOUNT]) && $_POST[CodupWooLoyalty::OPT_CATEGORY_MAX_POINT_DISCOUNT] >= 0  && is_numeric($_POST[CodupWooLoyalty::OPT_CATEGORY_MAX_POINT_DISCOUNT]))
            {
                update_term_meta($term_id, CodupWooLoyalty::OPT_CATEGORY_MAX_POINT_DISCOUNT, $_POST[CodupWooLoyalty::OPT_CATEGORY_MAX_POINT_DISCOUNT]);
            }
            else if($_POST[CodupWooLoyalty::OPT_CATEGORY_MAX_POINT_DISCOUNT]=='' || empty($_POST[CodupWooLoyalty::OPT_CATEGORY_MAX_POINT_DISCOUNT]))
            {
                update_term_meta($term_id, CodupWooLoyalty::OPT_CATEGORY_MAX_POINT_DISCOUNT, 0);
            }
            else
            {
                WC_Admin_Meta_Boxes::add_error('Invalid Data Entered In Maximum Discount');
            }
        }
        else
        {
            $tier_array = $_POST[CodupWooLoyalty::OPT_CATEGORY_DISCOUNT_TIERS];
            $valid = true;
            if($tier_array != NULL)
            {
                foreach($tier_array as $index => $tier) 
                {
                    if(isset($tier['from']) && isset($tier['to']) && isset($tier['discount']))
                    {
                        if( is_numeric($tier['from']) && $tier['from'] >= 0 && 
                            is_numeric($tier['to']) && $tier['to'] >= 0 &&
                            is_numeric($tier['discount']) && $tier['discount'] >= 0
                        )
                        {
                            if($valid)
                            {
                                if($tier['from'] > $tier['to']){
                                    $valid = false;
                                    WC_Admin_Meta_Boxes::add_error(sprintf(__('"Form" must be less than "To" in Tier %d!', CWL_SLUG), $index+1));
                                }
                            }
                        }
                        else
                        {
                            WC_Admin_Meta_Boxes::add_error('Invalid Data Entered In Tiers Field<br>');
                            $valid = false;
                        }
                    }
                }
                if($valid)
                {
                    $currSymbol = get_woocommerce_currency_symbol();
                    $overlaps = array();
                    // check if any tiers overlap
                    foreach($tier_array as $index => $tier)
                    {
                        foreach($tier_array as $otherIndex => $otherTier)
                        {
                            $pairId = $index.'-'.$otherIndex;
                            if($index != $otherIndex && !in_array($pairId, $overlaps))
                            {
                                if($tier['from'] >= $otherTier['from'] && $tier['from'] <= $otherTier['to'] ||
                                    $tier['to'] >= $otherTier['from'] && $tier['to'] <= $otherTier['to']
                                )
                                {
                                    $valid = false;
                                    WC_Admin_Meta_Boxes::add_error(sprintf(
                                        __('Tier %s (%s - %s) overlaps with Tier %s (%s - %s) in the range %s - %s. Please make sure these tiers do not overlap!', CWL_SLUG), 
                                        $otherIndex+1,
                                        $currSymbol.$otherTier['from'],
                                        $currSymbol.$otherTier['to'],
                                        $index+1,
                                        $currSymbol.$tier['from'],
                                        $currSymbol.$tier['to'],
                                        $currSymbol.max($tier['from'], $otherTier['from']),
                                        $currSymbol.min($tier['to'], $otherTier['to'])
                                    ));
                                    
                                    $overlaps[] = $pairId;
                                    $overlaps[] = $otherIndex.'-'.$index;
                                }
                            }
                            else if(in_array($pairId, $overlaps))
                            {
                                $valid = false;
                            }
                        }
                    }
                }
                
                if($valid){
                    // keep discount at max 100
                    foreach($tier_array as $key => &$tier){
                        if($tier['discount'] > 100){
                            $tier['discount'] = 100;
                        }
                    }
                    update_term_meta($term_id, CodupWooLoyalty::OPT_CATEGORY_DISCOUNT_TIERS, array_values($tier_array));
                }
            }
        }
    }

    /**
     * Adds the content of the CWL custom fields in WC category page.
     */
    public function addCustomFieldsOnCatPage()
    {
        $mode = get_option(CodupWooLoyalty::OPT_DISCOUNT_MODE, CodupWooLoyalty::MODE_POINTS);
        if($mode == CodupWooLoyalty::MODE_POINTS)
        {
            $this->getCategoryPointsModeFields();
        }
        else
        {
            woocommerce_admin_fields($this->getCategoryTiersModeFields());
        }

    }
    
    /**
     * Returns the setting fields for the points based discount mode.
     */
    public function getCategoryPointsModeFields()
    {
        $point_field_id = CodupWooLoyalty::OPT_CATEGORY_POINTS_VALUE;
        $max_discount_field_id = CodupWooLoyalty::OPT_CATEGORY_MAX_POINT_DISCOUNT;
        ?>
        <div class="form-field">
            <label for=<?php echo $point_field_id ?>><?php _e( 'Point Value ', '' );echo'('.get_woocommerce_currency_symbol().')'; ?></label>
            <input type="text" name=<?php echo $point_field_id ?> id=<?php echo $point_field_id ?> value="">
            <?php _e( 'How much money needs to be spent to receive a single point.<br><br>', '' ); ?>
            
            <label for=<?php echo $max_discount_field_id ?>><?php _e( 'Maximum Discount (%)', '' ); ?></label>
            <input type="text" name=<?php echo $max_discount_field_id ?> id=<?php echo $max_discount_field_id ?> value="">
            <?php _e( 'The maximum amount if discount the user can get using points.', '' ); ?>
        </div>
        
        <?php
    }

    /**
     * Returns the setting fields for the points based discount mode on Wocommerce edit category page.
     */
    public function getEditPageCategoryPointsModeFields($term)
    {

        $point_field_id = CodupWooLoyalty::OPT_CATEGORY_POINTS_VALUE;
        $max_discount_field_id = CodupWooLoyalty::OPT_CATEGORY_MAX_POINT_DISCOUNT;
        $term_meta = get_term_meta($term->term_id);

        ?>
        <tr class="form-field">
            <th scope="row" valign="top">
                <label for=<?php echo $point_field_id ?>><?php _e( 'Point Value', '' ); ?></label>
            </th>
            <td>
                <input type="text" name=<?php echo $point_field_id ?> value="<?php echo esc_attr( $term_meta[$point_field_id][0] ) ? esc_attr( $term_meta[$point_field_id][0] ) : ''; ?>"><span><?php echo ' '.get_woocommerce_currency_symbol(); ?></span>
                <?php _e( 'How much money needs to be spent to receive a single point.', '' ); ?>
            </td>
        </tr>
        <tr class="form-field">
            <th scope="row" valign="top">
                <label for=<?php echo $max_discount_field_id ?>><?php _e( 'Maximum Discount', '' ); ?></label>
            </th>
            <td>
                <input type="text" name=<?php echo $max_discount_field_id ?> value="<?php echo esc_attr( $term_meta[$max_discount_field_id][0] ) ? esc_attr( $term_meta[$max_discount_field_id][0] ) : ''; ?>">
                <?php _e( '% The maximum amount if discount the user can get using points.', '' ); ?>
            </td>
        </tr>
        
        <?php
    
    }

    /**
     * Returns the setting fields for the tiers based discount mode.
     * @return mixed[]
     */
    public function getCategoryTiersModeFields()
    {
        $term = $_GET['tag_ID'];
        $fields = array(
            'tbds_title' => array(
                'title' => __('Discount Tiers', CWL_SLUG),
                'type' => 'title',
                'desc' => __('Please enter the ranges of money the user needs to '.
                            'have spent within the discount lifespan to get the '. 
                            'discounts in the respective tiers.<br/>', CWL_SLUG), 
                'id' => 'wc_settings_tab_'.CWL_SLUG.'_tbds_title',
            ),
            
            'tier_header' => array(
                'type' => CodupWooLoyaltyTiersFields::FIELD_HEADER_TYPE,
                'id' => 'cwl-tiers-header'
            ),
        );
        $tiers = get_term_meta($term,'wc_settings_tab_codup_woo_loyalty_category_discount_tiers',true);

        if($tiers == null || count($tiers) == 0){
            $tiers = array(array('from' => '', 'to' => '', 'discount' => ''));
        }
        $fields['tiers'] = array(
            'title' => __('Tier %d', CWL_SLUG),
            'type' => CodupWooLoyaltyTiersFields::FIELD_TYPE,
            'id' => CodupWooLoyalty::OPT_CATEGORY_DISCOUNT_TIERS,
            'class' => 'cwl-tiers-mode',
            'tiers' => $tiers
        );
        
        $fields['tier_footer'] = array(
            'type' => CodupWooLoyaltyTiersFields::FIELD_FOOTER_TYPE,
            'id' => 'cwl-tiers-footer'
        );
        $fields['tbds_section_end'] = array(
            'type' => 'sectionend',
            'id' => 'wc_settings_tab_'.CWL_SLUG.'_tbds_title',
        );
        
        return $fields;
    }

    //---------------------------SPIRINT 3 : Support Product Specific Discount-----------------------------------------//
    
    public function codup_woo_loyalty_tab_fields()
    {
        include 'templates/cwl_product_tab_fields.php';
    }

    public function codup_woo_loyalty_tab()
    {
        include 'templates/cwl_product_tab.php';
    }
    
    /**
     * save the product page custom fields for the tiers based discount mode.
     * save the product page custom fields for the points based discount mode.
     * @param $term_id getting the id of product 
     */
    public function saveCustomFieldsOnProductPage() 
    {
        global $post;
        $post_id = get_the_ID();
        // Save the taxonomy value.            
        $post_meta = get_post_meta($post_id);
        $mode = get_option(CodupWooLoyalty::OPT_DISCOUNT_MODE, CodupWooLoyalty::MODE_POINTS);
        
        if($mode == CodupWooLoyalty::MODE_POINTS)
        {    
            if(isset($_POST[CodupWooLoyalty::OPT_PRODUCT_POINTS_VALUE]) && $_POST[CodupWooLoyalty::OPT_PRODUCT_POINTS_VALUE] >= 0 && is_numeric($_POST[CodupWooLoyalty::OPT_PRODUCT_POINTS_VALUE]))
            {
                update_post_meta($post_id, CodupWooLoyalty::OPT_PRODUCT_POINTS_VALUE, $_POST[CodupWooLoyalty::OPT_PRODUCT_POINTS_VALUE]);
            }
            else if($_POST[CodupWooLoyalty::OPT_PRODUCT_POINTS_VALUE]=='' || empty($_POST[CodupWooLoyalty::OPT_PRODUCT_POINTS_VALUE]))
            {   
                update_post_meta($post_id, CodupWooLoyalty::OPT_PRODUCT_POINTS_VALUE, 0);
            }
            else
            {
                WC_Admin_Meta_Boxes::add_error('Invalid Data Entered In Points Value');
            }
            if(isset($_POST[CodupWooLoyalty::OPT_PRODUCT_MAX_POINT_DISCOUNT]) && $_POST[CodupWooLoyalty::OPT_PRODUCT_MAX_POINT_DISCOUNT] >= 0 && is_numeric($_POST[CodupWooLoyalty::OPT_PRODUCT_MAX_POINT_DISCOUNT]))
            {
                update_post_meta($post_id, CodupWooLoyalty::OPT_PRODUCT_MAX_POINT_DISCOUNT,$_POST[CodupWooLoyalty::OPT_PRODUCT_MAX_POINT_DISCOUNT]);
            }
            else if($_POST[CodupWooLoyalty::OPT_PRODUCT_MAX_POINT_DISCOUNT]=='' || empty($_POST[CodupWooLoyalty::OPT_PRODUCT_MAX_POINT_DISCOUNT]))
            {   
                update_post_meta($post_id, CodupWooLoyalty::OPT_PRODUCT_MAX_POINT_DISCOUNT, 0);
            }
            else
            {
                WC_Admin_Meta_Boxes::add_error('Invalid Data Entered In Maximum Discount');
            }
        }
        else
        {
            $valid = true;
            $tier_array = $_POST[CodupWooLoyalty::OPT_PRODUCT_DISCOUNT_TIERS];
            if($tier_array != NULL)
            {
                foreach($tier_array as $index => $tier) 
                {
                    if(isset($tier['from']) && isset($tier['to']) && isset($tier['discount']))
                    {
                        if( is_numeric($tier['from']) && $tier['from'] >= 0 && 
                            is_numeric($tier['to']) && $tier['to'] >= 0 &&
                            is_numeric($tier['discount']) && $tier['discount'] >= 0
                        )
                        {
                            if($valid)
                            {
                                if($tier['from'] > $tier['to'])
                                {
                                    $valid = false;
                                    WC_Admin_Meta_Boxes::add_error(sprintf(__('"From" must be less than "To" in Tier %d!', CWL_SLUG), $index+1));
                                }
                            }
                        }
                        else
                        {
                            WC_Admin_Meta_Boxes::add_error('Invalid Data Entered In Tiers Field');
                            $valid = false;
                        }
                    }
                }
                if($valid){
                    $currSymbol = get_woocommerce_currency_symbol();
                    $overlaps = array();
                    // check if any tiers overlap
                    foreach($tier_array as $index => $tier)
                    {
                        foreach($tier_array as $otherIndex => $otherTier)
                        {
                            $pairId = $index.'-'.$otherIndex;
                            if($index != $otherIndex && !in_array($pairId, $overlaps))
                            {
                                if($tier['from'] >= $otherTier['from'] && $tier['from'] <= $otherTier['to'] ||
                                    $tier['to'] >= $otherTier['from'] && $tier['to'] <= $otherTier['to']
                                )
                                {
                                    $valid = false;
                                    WC_Admin_Meta_Boxes::add_error(sprintf(
                                        __('Tier %s (%s - %s) overlaps with Tier %s (%s - %s) in the range %s - %s. Please make sure these tiers do not overlap!', CWL_SLUG), 
                                        $otherIndex+1,
                                        $currSymbol.$otherTier['from'],
                                        $currSymbol.$otherTier['to'],
                                        $index+1,
                                        $currSymbol.$tier['from'],
                                        $currSymbol.$tier['to'],
                                        $currSymbol.max($tier['from'], $otherTier['from']),
                                        $currSymbol.min($tier['to'], $otherTier['to'])
                                    ));
                                    
                                    $overlaps[] = $pairId;
                                    $overlaps[] = $otherIndex.'-'.$index;
                                }
                            }
                            else if(in_array($pairId, $overlaps))
                            {
                                $valid = false;
                            }
                        }
                    }
                }
                
                if($valid){
                    // keep discount at max 100
                    $tier_array = &$tier_array;
                    foreach($tier_array as $key => &$tier){
                        if($tier['discount'] > 100){
                            $tier['discount'] = 100;
                        }
                    }
                    update_post_meta($post_id,CodupWooLoyalty::OPT_PRODUCT_DISCOUNT_TIERS, array_values($tier_array));
                }
            }
        }
    }

    /**
     * Adds the content of the CWL custom fields in WC product page.
     */
    public function addCustomFieldsOnProductPage()
    {
        $mode = get_option(CodupWooLoyalty::OPT_DISCOUNT_MODE, CodupWooLoyalty::MODE_POINTS);
        if($mode == CodupWooLoyalty::MODE_POINTS)
        {
            $this->getEditPageProductPointsModeFields();
        }
        else
        {
            woocommerce_admin_fields($this->getProductTiersModeFields());
        }

    }

    /**
     * Returns the setting fields for the points based discount mode on Wocommerce product page.
     */
    public function getEditPageProductPointsModeFields()
    {
        global $post;
        
        $point_field_id = CodupWooLoyalty::OPT_PRODUCT_POINTS_VALUE;
        $max_discount_field_id = CodupWooLoyalty::OPT_PRODUCT_MAX_POINT_DISCOUNT;
        $post_meta = get_post_meta(get_the_ID());
        ?>
        <tr class="form-field">
            <th scope="row" valign="top">
                <label style="font-size:15px;font-weight: 400;"><?php _e( 'Points Based Discount', '' ); ?></label><br><br>
            </th>
        </tr>
        <tr class="form-field">
            <th scope="row" valign="top">
                <label for=<?php echo $point_field_id ?>><?php _e( 'Point Value ', '' );echo'('.get_woocommerce_currency_symbol().')'; ?></label>
            </th>
            <td>
                <input type="text" name=<?php echo $point_field_id ?> value="<?php echo esc_attr((int)$post_meta[$point_field_id][0] ) ? esc_attr( $post_meta[$point_field_id][0] ) : ''; ?>">
                <?php echo wc_help_tip('How much money needs to be spent to receive a single point.', '' ); ?><br><br>
            </td>
        </tr>
        
        <tr class="form-field">
            <th scope="row" valign="top">
                <label for=<?php echo $max_discount_field_id ?>><?php _e( 'Maximum Discount (%)', '' ); ?></label>
            </th>
            <td>
                <input type="text" name=<?php echo $max_discount_field_id ?> value="<?php echo esc_attr((int)$post_meta[$max_discount_field_id][0] ) ? esc_attr( $post_meta[$max_discount_field_id][0] ) : ''; ?>">
                <?php echo wc_help_tip('The maximum amount if discount the user can get using points.', '' ); ?>
            </td>
        </tr>
        <?php
    }

    /**
     * Returns the setting fields for the tiers based discount mode on Wocommerce product page.
     * @return mixed[]
     */
    public function getProductTiersModeFields()
    {
        $post = get_the_ID();
        $fields = array(
            'tbds_title' => array(
                'title' => __('Tiers Based Discont', CWL_SLUG),
                'type' => 'title',
                'desc' => __('Please enter the ranges of money the user needs to '.
                            'have spent within the discount lifespan to get the '. 
                            'discounts in the respective tiers.<br/>', CWL_SLUG), 
                'id' => 'wc_settings_tab_'.CWL_SLUG.'_tbds_title',
            ),
            
            'tier_header' => array(
                'type' => CodupWooLoyaltyTiersFields::FIELD_HEADER_TYPE,
                'id' => 'cwl-tiers-header'
            ),
        );
        $tiers = get_post_meta($post,CodupWooLoyalty::OPT_PRODUCT_DISCOUNT_TIERS,true);

        if($tiers == null || count($tiers) == 0)
        {
            $tiers = array(array('from' => '', 'to' => '', 'discount' => ''));
        }
        $fields['tiers'] = array(
            'title' => __('Tier %d', CWL_SLUG),
            'type' => CodupWooLoyaltyTiersFields::FIELD_TYPE,
            'id' => CodupWooLoyalty::OPT_PRODUCT_DISCOUNT_TIERS,
            'class' => 'cwl-tiers-mode',
            'tiers' => $tiers
        );
        
        $fields['tier_footer'] = array(
            'type' => CodupWooLoyaltyTiersFields::FIELD_FOOTER_TYPE,
            'id' => 'cwl-tiers-footer'
        );
        $fields['tbds_section_end'] = array(
            'type' => 'sectionend',
            'id' => 'wc_settings_tab_'.CWL_SLUG.'_tbds_title',
        );
        
        return $fields;
    }

    /**
     * Add links to the bottom of the plugin name in the WP plugins list
     * @param array $links
     */
    public function addActionLinks($links)
    {
        $cwlLinks = array(
            '<a href="'.admin_url('admin.php?page=wc-settings&tab=codup_woo_loyalty').'">Settings</a>'
        );
        return array_merge($cwlLinks, $links);
    }
}