<?php

/**
 * Tests generic Codup Woo Loyalty functionality.
 */
class TiersTest extends WC_Unit_Test_Case {
    public function testIfUserSpentCalulcatesCorrectly(){
        $user = $this->factory->user->create_and_get();
        $this->makePurchases($user, array(110, 332));
        
        $tierManager = new CodupWooLoyaltyTiers();
        
        $this->assertEquals(442, $tierManager->getUserHasSpentInDiscountLifespan($user->ID));
    }
    
    public function testIfUserSpentTakesDIscountLifetimeInAccount(){
        $user = $this->factory->user->create_and_get();
        $this->makePurchases($user, array(110, 332));
        
        $default = get_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN);
        update_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN, 1);
        
        $order = WC_Helper_Order::create_order($user->ID);
        $order->set_total(77);
        $order->set_status('wc-completed');
        $order->save();
        /* @var $post WP_Post */
        $post = get_post($order->get_id());
        $post->post_date = date('Y-m-d', time()-5*86400);
        wp_update_post($post);
        $post = get_post($order->get_id());
        
        $tierManager = new CodupWooLoyaltyTiers();
        $this->assertEquals(442, $tierManager->getUserHasSpentInDiscountLifespan($user->ID));
        
        update_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN, $default);
    }
    
    public function testIfRightTierIsChosen(){
        $user = $this->factory->user->create_and_get();
        $this->makePurchases($user, array(110, 332));
        
        $defaultLifespan = get_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN);
        $defaultIers = get_option(CodupWooLoyalty::OPT_DISCOUNT_TIERS);
        update_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN, 1);
        update_option(CodupWooLoyalty::OPT_DISCOUNT_TIERS, array(array(
            'from' => 0,
            'to' => 200,
            'discount' => 10
        ),array(
            'from' => 201,
            'to' => 400,
            'discount' => 20
        ),array(
            'from' => 401,
            'to' => 600,
            'discount' => 30
        )));
        
        $order = WC_Helper_Order::create_order($user->ID);
        $order->set_total(77);
        $order->set_status('wc-completed');
        $order->save();
        /* @var $post WP_Post */
        $post = get_post($order->get_id());
        $post->post_date = date('Y-m-d', time()-5*86400);
        wp_update_post($post);
        $post = get_post($order->get_id());
        
        $tierManager = new CodupWooLoyaltyTiers();
        $this->assertEquals(30, $tierManager->getUserTierDiscount($user->ID));
        
        update_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN, $defaultLifespan);
        update_option(CodupWooLoyalty::OPT_DISCOUNT_TIERS, $defaultIers);
    }
    
    public function testIf0DicountIfNoTiearMatches(){
        $user = $this->factory->user->create_and_get();
        $this->makePurchases($user, array(110, 332, 2000));
        
        $defaultLifespan = get_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN);
        $defaultIers = get_option(CodupWooLoyalty::OPT_DISCOUNT_TIERS);
        update_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN, 1);
        update_option(CodupWooLoyalty::OPT_DISCOUNT_TIERS, array(array(
            'from' => 0,
            'to' => 200,
            'discount' => 10
        ),array(
            'from' => 201,
            'to' => 400,
            'discount' => 20
        ),array(
            'from' => 401,
            'to' => 600,
            'discount' => 30
        )));
        
        $order = WC_Helper_Order::create_order($user->ID);
        $order->set_total(77);
        $order->set_status('wc-completed');
        $order->save();
        /* @var $post WP_Post */
        $post = get_post($order->get_id());
        $post->post_date = date('Y-m-d', time()-5*86400);
        wp_update_post($post);
        $post = get_post($order->get_id());
        
        $tierManager = new CodupWooLoyaltyTiers();
        $this->assertEquals(0, $tierManager->getUserTierDiscount($user->ID));
        
        update_option(CodupWooLoyalty::OPT_DISCOUNT_LIFESPAN, $defaultLifespan);
        update_option(CodupWooLoyalty::OPT_DISCOUNT_TIERS, $defaultIers);
    }
    
    /**
     * Creates completes WC purchases for the specific user, using the values in
     * $otoals array.
     * @param WP_User $user
     * @param int[] $totals the total prices to use
     */
    private function makePurchases($user, $totals){
        foreach($totals AS $total){
            $order = WC_Helper_Order::create_order($user->ID);
            $order->set_total($total);
            $order->set_status('wc-completed');
            $order->save();
        }
    }
}
